﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Controls.Primitives;

namespace MoreControls
{
    /// <summary>
    /// Displays a "live image" video thumbnail that plays the video on mouse over and stops playing
    /// when the mouse moves out.
    /// The control can be exclusively selected (similar to RadioButton)
    /// It contains Title, Description, the Video and it's associated Preview image as well as 
    /// Content below the video
    /// </summary>
    [TemplatePart(Name = "liveImage", Type = typeof(LiveImage))]
    [TemplatePart(Name = "rootPanel", Type = typeof(Panel))]
    
    [TemplateVisualState(Name = "Normal", GroupName = "CommonStates")]
    [TemplateVisualState(Name = "MouseOver", GroupName = "CommonStates")]
    [TemplateVisualState(Name = "Pressed", GroupName = "CommonStates")]
    
    [TemplateVisualState(Name = "Unchecked", GroupName = "CheckStates")]
    [TemplateVisualState(Name = "Checked", GroupName = "CheckStates")]
    public class LiveImageInfoRadioButton : RadioButton
    {
        public LiveImageInfoRadioButton()
        {
            DefaultStyleKey = typeof(LiveImageInfoRadioButton);
            base.Checked += new RoutedEventHandler(LiveImageInfoRadioButton_Checked);
            base.Unchecked += new RoutedEventHandler(LiveImageInfoRadioButton_Unchecked);
        }

        protected override void OnToggle()
        {
            base.OnToggle();
            VisualStateManager.GoToState(this, "Checked", true);
        }

        void LiveImageInfoRadioButton_Unchecked(object sender, RoutedEventArgs e)
        {
            VisualStateManager.GoToState(this, "Unchecked", true);
        }

        void LiveImageInfoRadioButton_Checked(object sender, RoutedEventArgs e)
        {
            VisualStateManager.GoToState(this, "Checked", true);
        }

        protected override void OnMouseEnter(MouseEventArgs e)
        {
            base.OnMouseEnter(e);
            VisualStateManager.GoToState(this, "MouseOver", true);
        }

        protected override void OnMouseLeave(MouseEventArgs e)
        {
            base.OnMouseLeave(e);
            VisualStateManager.GoToState(this, "Normal", true);
        }

        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonDown(e);
            VisualStateManager.GoToState(this, "Pressed", true);
        }

        protected override void OnMouseLeftButtonUp(MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonUp(e);
            IsChecked = true;
            VisualStateManager.GoToState(this, "MouseOver", true);
        }

        protected override void OnGotFocus(RoutedEventArgs e)
        {
            base.OnGotFocus(e);
            VisualStateManager.GoToState(this, "Focused", true);
        }

        protected override void OnLostFocus(RoutedEventArgs e)
        {
            base.OnLostFocus(e);
            VisualStateManager.GoToState(this, "Unfocused", true);
        }

        void LiveImageInfoRadioButton_MouseEnter(object sender, MouseEventArgs e)
        {
            if (_liveImagePart == null) return;
            System.Diagnostics.Debug.WriteLine("Play in LiveImageInfoRadioButton_MouseEnter");
            _liveImagePart.Play();
        }
        //Storyboard _hover = null;

        void LiveImageInfoRadioButton_MouseLeave(object sender, MouseEventArgs e)
        {
            if (_liveImagePart == null) return;
            _liveImagePart.Pause();
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            _liveImagePart = (LiveImage)GetTemplateChild("liveImage");
            RootPanel = (Panel)GetTemplateChild("rootPanel");
            //_hover = (Storyboard) RootPanel.Resources["Hover"];
        }

        private LiveImage _liveImagePart = null;
        private Panel _rootPanel = null;

        /// <summary>
        /// Contains the root panel defined in the template
        /// </summary>
        private Panel RootPanel
        {
            get
            {
                return _rootPanel;
            }

            set
            {
                Panel old = _rootPanel;

                if (old != null)
                {
                    old.MouseEnter -= new MouseEventHandler(LiveImageInfoRadioButton_MouseEnter);
                    old.MouseLeave -= new MouseEventHandler(LiveImageInfoRadioButton_MouseLeave);
                }

                _rootPanel = value;

                if (_rootPanel != null)
                {
                    _rootPanel.MouseEnter += new MouseEventHandler(LiveImageInfoRadioButton_MouseEnter);
                    _rootPanel.MouseLeave += new MouseEventHandler(LiveImageInfoRadioButton_MouseLeave);
                }
            }
        }

        #region Public Properties
        public static readonly DependencyProperty TitleProperty =
            DependencyProperty.Register("Title", typeof(string), typeof(LiveImageInfoRadioButton),
            new PropertyMetadata(new PropertyChangedCallback(LiveImageInfoRadioButton.OnPropertyChanged)));

        /// <summary>
        /// Title of the live video
        /// </summary>
        public string Title
        {
            get { return (string)GetValue(TitleProperty); }
            set { SetValue(TitleProperty, value); }
        }

        public static readonly DependencyProperty VideoProperty =
            DependencyProperty.Register("Video", typeof(string), typeof(LiveImageInfoRadioButton),
            new PropertyMetadata(new PropertyChangedCallback(LiveImageInfoRadioButton.OnPropertyChanged)));

        /// <summary>
        /// Location of the video file (e.g. nokola.com/video.wmv)
        /// </summary>
        public string Video
        {
            get { return (string)GetValue(VideoProperty); }
            set { SetValue(VideoProperty, value); }
        }

        public static readonly DependencyProperty PreviewProperty =
            DependencyProperty.Register("Preview", typeof(string), typeof(LiveImageInfoRadioButton),
            new PropertyMetadata(new PropertyChangedCallback(LiveImageInfoRadioButton.OnPropertyChanged)));

        /// <summary>
        /// The video preview picture to display before the video is loaded
        /// Usually this is the first frame of the video or some other key picture 
        /// representing the video
        /// </summary>
        public string Preview
        {
            get { return (string)GetValue(PreviewProperty); }
            set { SetValue(PreviewProperty, value); }
        }

        public static readonly DependencyProperty DescriptionProperty =
            DependencyProperty.Register("Description", typeof(string), typeof(LiveImageInfoRadioButton),
            new PropertyMetadata(new PropertyChangedCallback(LiveImageInfoRadioButton.OnPropertyChanged)));

        /// <summary>
        /// Description of the video (usually 1 or 2 sentences)
        /// </summary>
        public string Description
        {
            get { return (string)GetValue(DescriptionProperty); }
            set { SetValue(DescriptionProperty, value); }
        }

        #endregion


        private static void OnPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
        }
    }
}
