﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;

namespace MoreControls
{
    /// <summary>
    /// Displays an image that smoothly transitions to another image once its Source is changed
    /// Useful for slideshows and places where we need to show progressive image changes
    /// Cool samples can be: "grow a beard" scenario - one picture without a beard transitions 
    /// to one with a beard; or "turning day to night" scenario
    /// </summary>
    public partial class SmoothImage : UserControl
    {
        private string _source = null;
        Image newImage;
        BitmapImage bi;
        Storyboard sb = new Storyboard();
        TimeSpan _animDuration = TimeSpan.FromSeconds(0.5);

        public string Source
        {
            get
            {
                return _source;
            }

            set
            {
                newImage = new Image();
                newImage.ImageFailed += new EventHandler<ExceptionRoutedEventArgs>(newImage_ImageFailed);
                bi = new BitmapImage();
                bi.UriSource = new Uri(value, UriKind.RelativeOrAbsolute);
                bi.DownloadProgress += new EventHandler<DownloadProgressEventArgs>(bi_DownloadProgress);
                newImage.Source = bi;
                newImage.Opacity = 0;
                newImage.Stretch = Stretch.UniformToFill;
                rootGrid.Children.Add(newImage);
            }
        }

        void bi_DownloadProgress(object sender, DownloadProgressEventArgs e)
        {
            if (e.Progress < 100)
            {
                return;
            }
            
            // the image loaded, set the source
            Image image = (Image)newImage;
            
            // check if image is updated and ignore request in such case
            // this can happen if the source is changed in the middle of image download
            if (newImage.Source != sender)
            {
                return;
            }

            if (_source == null) // if first image being displayed, no need to animate
            {
                _source = ((BitmapImage)image.Source).UriSource.OriginalString;
                image.Opacity = 1;
                return;
            }
            _source = ((BitmapImage)image.Source).UriSource.OriginalString;

            // fade in the new image
            
            // check if we have other animation running and remove its completed handler
            // to avoid deleting controls before our current "top" animation is done
            if (sb != null) {
                sb.Completed -= sb_Completed;
            }
            sb = new Storyboard();
            DoubleAnimation anim = new DoubleAnimation();
            anim.To = 1;
            anim.Duration = _animDuration;
            Storyboard.SetTarget(anim, image);
            Storyboard.SetTargetProperty(anim, new PropertyPath("Opacity"));
            sb.Children.Add(anim);
            sb.Completed += new EventHandler(sb_Completed);
            sb.Begin();
        }

        void newImage_ImageFailed(object sender, ExceptionRoutedEventArgs e)
        {
            // image failed to load, remove it
            if (rootGrid.Children.Count > 0)
            {
                rootGrid.Children.RemoveAt(rootGrid.Children.Count - 1);
            }
        }

        void sb_Completed(object sender, EventArgs e)
        {
            // the image is displayed, cleanup other child images
            for (int i = rootGrid.Children.Count - 2; i >= 0; i--)
            {
                rootGrid.Children.RemoveAt(i);
            }
        }

        public SmoothImage()
        {
            InitializeComponent();
        }
    }
}
